
;*******************************************************************************
;            void I2C_Init(uint32_t i2c_clk_freq)
;*******************************************************************************
	
I2C_Init:
	MOVLW	0x28	; SSP1CON1  = 0x28; configure MSSP module to work in I2C mode
	MOVLB	0x3
	MOVWF	SSP1CON1
	MOVLW	0x4F	; SSP1ADD  = 0x4F; set I2C clock frequency
	MOVWF	SSP1ADD	; (_XTAL_FREQ /(4 * i2c_clk_freq))-1 = (32000000Mhz/(4*100000khz) -1 = 79 = 0x4F
	CLRF	SSP1STAT; SSP1STAT = 0;
	RETURN
	
;*******************************************************************************             
;           void I2C_Start()
;******************************************************************************* 
	
I2C_Start:	
	MOVLB	0x3		; while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));
	BTFSC	SSP1STAT, 0x2	; wait for MSSP module to be free (not busy)
	GOTO	I2C_Start
	MOVF	SSP1CON2, W
	ANDLW	0x1F
	BTFSS	STATUS, 0x2
	GOTO	I2C_Start
	BSF	SSP1CON2, 0x0	; SEN1 = 1;  // initiate start condition
	RETURN
;******************************************************************************* 
;       void I2C_Repeated_Start()
;******************************************************************************* 
  
I2C_Repeated_Start:
	
ACA4:	
	MOVLB	0x3		; while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));
	BTFSC	SSP1STAT, 0x2	; wait for MSSP module to be free (not busy)
	GOTO	ACA4
	MOVF	SSP1CON2, W
	ANDLW	0x1F
	BTFSS	STATUS, 0x2
	GOTO	ACA4
	BSF	SSP1CON2, 0x1	; RSEN1 = 1;  // initiate repeated start condition
	RETURN
;*******************************************************************************          
;          void I2C_Stop()
;*******************************************************************************
	
I2C_Stop:
  
ACB6:	
	MOVLB	0x3		; while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));
	BTFSC	SSP1STAT, 0x2	; wait for MSSP module to be free (not busy)
	GOTO	ACB6
	MOVF	SSP1CON2, W
	ANDLW	0x1F
	BTFSS	STATUS, 0x2
	GOTO	ACB6
	BSF	SSP1CON2, 0x2	; PEN1 = 1;  // initiate stop condition
	RETURN
;*******************************************************************************           
;           void I2C_Write(uint8_t i2c_data)
;*******************************************************************************
I2C_Write:
	MOVWF	0x1F0
ACCB:	
	MOVLB	0x3		; while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F)); 
	BTFSC	SSP1STAT, 0x2	; wait for MSSP module to be free (not busy)
	GOTO	ACCB
	MOVF	SSP1CON2, W
	ANDLW	0x1F
	BTFSS	STATUS, 0x2
	GOTO	ACCB
	MOVF	0x1F0, W
	MOVWF	SSP1BUF		; SSP1BUF = i2c_data;  // update buffer
	RETURN
	
;*******************************************************************************           
;          uint8_t I2C_Read(uint8_t ack)
;*******************************************************************************           

I2C_Read:
	MOVWF	0x1F1
AD70:	
	MOVLB	0x3		; while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));
	BTFSC	SSP1STAT, 0x2	; wait for MSSP module to be free (not busy)
	GOTO	AD70
	MOVF	SSP1CON2, W
	ANDLW	0x1F
	BTFSS	STATUS, 0x2
	GOTO	AD70
	BSF	SSP1CON2, 0x3	; RCEN1 = 1;
; while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F))
AD78:	;wait for MSSP module to be free (not busy)
	BTFSC	SSP1STAT, 0x2
	GOTO	AD78
	MOVF	SSP1CON2, W
	ANDLW	0x1F
	BTFSS	STATUS, 0x2
	GOTO	AD78
; _data = SSP1BUF; read data from buffer
	MOVF	SSP1BUF, W
	MOVWF	0x1F0
	MOVF	0x1F0, W
	MOVWF	0x1F2
; while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F))
AD82:	; wait for MSSP module to be free (not busy)
	BTFSC	SSP1STAT, 0x2
	GOTO	AD82
	MOVF	SSP1CON2, W
	ANDLW	0x1F
	BTFSS	STATUS, 0x2
	GOTO	AD82
	MOVF	0x1F1, W; send acknowledge pulse ? (depends on ack, if 1 send, otherwise don't send)
	BTFSS	STATUS, 0x2	; ACKDT1 = !ack;
	GOTO	AD8E
	MOVLB	0x3
	BSF	SSP1CON2, 0x5
	GOTO	AD90
AD8E:	
	MOVLB	0x3
	BCF	SSP1CON2, 0x5
AD90:
	BSF	SSP1CON2, 0x4	; ACKEN1 = 1;
	MOVF	0x1F2, W	; return _data; return data read
	RETURN
;*******************************************************************************
;          // function for button debounce
;            __bit debounce (uint8_t button)
;*******************************************************************************	
debounce: 
	MOVWF	ButtonFlags	; Buttonflags
	CLRF	count		; uint8_t count = 0;
	CLRF	i		; for(uint8_t i = 0; i < 5; i++)
	GOTO	AE3B
AE33:
	INCF	i, F
	MOVLW	0x5
	SUBWF	i, W
	BTFSC	STATUS, 0x0
	GOTO	AE44
	GOTO	AE2B
AE3B:
	MOVF	ButtonFlags, W	; switch (button)
	XORLW	0x1
	BTFSC	STATUS, 0x2
	GOTO	AE20
	XORLW	0x2
	BTFSC	STATUS, 0x2
	GOTO	AE28
	GOTO	AE2B
AE20:
	MOVLB	0x0		; case 1: if (button1 == 0)
	BTFSC	PORTB, 0x1
	GOTO	AE2B
AE23:
	INCF	count, F	; count++;
	GOTO	AE2B
AE28:	
	MOVLB	0x0		; break;
	BTFSS	PORTB, 0x3	; case 3: if (button3 == 0)
	GOTO	AE23	
AE2B:
	CALL	wait245ms	;__delay_ms(10);
	GOTO	AE33
AE44:
	MOVLW	0x3		; if(count > 2)  return 1;
	SUBWF	count, W	; else return 0;
	RETURN

