
; File:     main.S
; Target:   PIC16F18677
; Author:   ARudi
; Date:     .........
; Compiler: pic-as(v2.20)
; IDE:      MPLABX v5.40
;
; Description:
;
; Add this line in the project properties box, pic-as Global Options -> Additional options: 
;    -Wl,-DCODE=2 -Wl,-pPor_Vec=0h,-pIsr_Vec=4h
;
;
PROCESSOR   16F18877
	
; PIC16F18877 Configuration Bit Settings

; CONFIG1
config FEXTOSC =    ECM   ; External Oscillator mode selection bits->Oscillator not enabled
config RSTOSC =	    HFINT1    ; Power-up default value for COSC bits->HFINTOSC (1MHz)
config CLKOUTEN =   OFF    ; Clock Out Enable bit->CLKOUT function is disabled; i/o or oscillator function on OSC2
config CSWEN =	    ON    ; Clock Switch Enable bit->Writing to NOSC and NDIV is allowed
config FCMEN =	    ON    ; Fail-Safe Clock Monitor Enable bit->FSCM timer enabled

; CONFIG2
config MCLRE =	    ON    ; Master Clear Enable bit->MCLR pin is Master Clear function
config PWRTE =	    OFF    ; Power-up Timer Enable bit->PWRT disabled
config LPBOREN =    OFF  ; Low-Power BOR enable bit->ULPBOR disabled
config BOREN =	    OFF    ; Brown-out reset enable bits->Brown-out Reset Enabled, SBOREN bit is ignored
config BORV =	    LO    ; Brown-out Reset Voltage Selection->Brown-out Reset Voltage (VBOR) set to 1.9V on LF, and 2.45V on F Devices
config ZCD =	    OFF    ; Zero-cross detect disable->Zero-cross detect circuit is disabled at POR.
config PPS1WAY =    ON    ; Peripheral Pin Select one-way control->The PPSLOCK bit can be cleared and set only once in software
config STVREN =	    ON    ; Stack Overflow/Underflow Reset Enable bit->Stack Overflow or Underflow will cause a reset

; CONFIG3
config WDTCPS = WDTCPS_31    ; WDT Period Select bits->Divider ratio 1:65536; software control of WDTPS
config WDTE =	OFF    ; WDT operating mode->WDT Disabled, SWDTEN is ignored
config WDTCWS = WDTCWS_7    ; WDT Window Select bits->window always open (100%); software control; keyed access not required
config WDTCCS = SC    ; WDT input clock selector->Software Control

; CONFIG4
config WRT =	OFF    ; UserNVM self-write protection bits->Write protection off
config SCANE =	available    ; Scanner Enable bit->Scanner module is available for use
config LVP =	ON    ; Low Voltage Programming Enable bit->Low Voltage programming enabled. MCLR/Vpp pin function is MCLR.

; CONFIG5
config CP =	OFF    ; UserNVM Program memory code protection bit->Program Memory code protection disabled
config CPD =	OFF    ; DataNVM code protection bit->Data EEPROM code protection disabled
	  
;*******************************************************************************
	
#include <xc.inc>

	
PSECT   udata
global	counter,i,ButtonFlags,loops,loops2,parameter,second,minute,hour,w_day,m_day
global	month,year,alarm1_minute,alarm1_hour,alarm2_minute,alarm2_hour
global	status_reg,alarm1_status,alarm2_status
	
	count:		DS      1               
	i:		DS      1 
	ButtonFlags:	DS	1
	loops:		DS	1
	loops2:		DS	1
	parameter:	DS	1
	second:		DS      1               
	minute:		DS      1
	hour:		DS      1
	w_day:		DS	1   
	m_day:		DS	1  
	month:		DS	1	
	year:		DS	1 
	alarm1_minute:	DS	1	
	alarm1_hour:	DS	1 
	alarm2_minute:	DS	1	
	alarm2_hour:	DS	1
	status_reg:	DS	1	
	alarm1_status:	DS	1 
	alarm2_status:	DS	1 

;*******************************************************************************	
PSECT udata_shr
 global __pcstackBANK0,al,row,number,__pcstackCOMMON,n,dividend,LCD_Char,x
 global	button,Date,col,sign,multiplier,product,quotient
 
	__pcstackBANK0:	DS      1               
	al:		DS      1 
	row:		DS	1
	number:		DS	1
	__pcstackCOMMON:DS	1
	n:		DS	1
	dividend:	DS      1               
	LCD_Char:	DS      1
	x:		DS      1
	button:		DS	1   
	Date:		DS	1  
	col:		DS	1	
	sign:		DS	1 
	multiplier:	DS	1   
	product:	DS	1  
	quotient:	DS	1
 
 
 
 
 
 
 
;*******************************************************************************	

#define		Button_1	ButtonFlags,1
#define		Button_3        ButtonFlags,3	; 1: Temperatur ist negativ
	

PSECT   Por_Vec,global,class=CODE,delta=2
 
resetVec:
	pagesel Start
	goto	Start
	
;PSECT   Isr_Vec,global,class=CODE,delta=2
    
;Isr_Vec:
	
Start:
	Time:
	;IRPC char,  :  :  
	;DB 'char'
	;ENDM
	
	
; Initialize the PIC hardware 
	MOVLB	0x0	; Bank 0
	CLRF    TRISA	; TRIS = 0 = Out
	MOVLW	0XFF
	MOVWF	TRISB
	CLRF    TRISC
	CLRF    TRISD
	CLRF    TRISE
	CLRF	LATA	; Data Latch Register
	CLRF	LATB
	CLRF	LATC
	CLRF	LATD
	CLRF	LATE			
	CLRF	PORTA	; 1 = Port pin is > V IH
	CLRF	PORTB	; 0 = Port pin is < V IL
	CLRF	PORTC
	CLRF	PORTD
	CLRF	PORTE			
	MOVLB   0x3E	; Bank 62
	CLRF    ANSELA	; 1 = Analog input. 
	CLRF    ANSELB	; 0 = Digital I/O. 
	CLRF    ANSELC
	CLRF    ANSELD
	CLRF    ANSELE			
	CLRF    WPUA	; 0= Pull-up disabled
	clrf	WPUB	; 1 = Pull-up enabled 
	CLRF    WPUC
	CLRF    WPUD
	CLRF    WPUE			
	CLRF    ODCONA	; 0 = standard push-pull drive (source and sink current)
	CLRF    ODCONB	; 1 = open-drain drive (sink current only) 
	CLRF    ODCONC
	CLRF    ODCOND
	CLRF	ODCONE
	MOVLW   0xFF	; 1 = Port pin slew rate is limited
	MOVWF	SLRCONA	; 0 = Port pin slews at maximum rate
	MOVWF   SLRCONB
	MOVWF   SLRCONC
	MOVWF   SLRCOND
	MOVLW   0x07
	MOVWF   SLRCONE			
	MOVLW   0xFF	; 1 = ST input used for PORT reads and interrupt-on-change
	MOVWF   INLVLA	; 0 = TTL input used for PORT reads and interrupt-on-change
	MOVWF   INLVLB
	MOVWF   INLVLC
	MOVWF   INLVLD
	MOVLW   0x0F
	MOVWF   INLVLE
	

;*******************************************************************************
;				main function
;*******************************************************************************
	
main:          
	MOVLW	0x3		; OSCFRQ = 0x03; set internal oscillator to 8MHz
	MOVLB	0x11
	MOVWF	OSCFRQ
	MOVLB	0x3E		; ANSELB = 0; configure all PORTB pins as digital
	CLRF	ANSELB
	MOVLB	0x0		; PORTB  = 0; PORTB initial state
	CLRF	PORTB
	MOVLW	0xF		; configure RB0 ~ 3 as input pins
	MOVWF	TRISB
	MOVLW	0xF		; enable RB0, RB1, RB2 and RB3 internal pull ups
	MOVLB	0x3E
	MOVWF	WPUB
	MOVLB	0xE		; INTE   = 0; clear external interrupt flag bit
	BCF	PIE0, 0x0
	BSF	INTCON, 0x7	; GIE    = 1; enable global interrupts
	BCF	INTCON, 0x0	; INTEDG = 0; set external Interrupt on falling edge
	BSF	PIE0, 0x0	; INTE   = 1; enable external interrupt     
	CALL	wait5ms		;__delay_ms(1000);  // wait 1 second  
	CALL	I2C_Init	; I2C_Init(100000); initialize I2C bus with clock frequency of 100kHz
	CALL	LCD_Begin	; initialize LCD module
	

	
;*******************************************************************************	
;++++++++++++++++++++++++++  RTC-Funktions   +++++++++++++++++++++++++++++++++++
;*******************************************************************************	
	
;*******************************************************************************	
;		uint8_t bcd_to_decimal(uint8_t number)
;*******************************************************************************	
	
bcd_to_decimal:	
	MOVWF	number
	MOVLW	0xF		;return((number >> 4) * 10 + (number & 0x0F));
	ANDWF	number, W
	MOVWF	0x74		;(number & 0x0F)
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x4
AAC:	
	LSRF	number, F
	DECFSZ	WREG, F	
	GOTO	AAC
	MOVF	number, W	; Wert(number >> 4)
	CALL	__bmul
	ADDWF	0x74, W
	RETURN
	
;*******************************************************************************		
;	convert decimal to BCD function
;	uint8_t decimal_to_bcd(uint8_t number)
;
;	return(((number / 10) << 4) + (number % 10));
;*******************************************************************************
	
decimal_to_bcd:
	
	MOVWF	number        
	MOVLW	10
	MOVWF	__pcstackCOMMON	;10
	MOVLW	0x0
	MOVWF	n
	MOVF	number, W
	MOVLB	0x0
	MOVWF	__pcstackBANK0
	CLRF	al
	MOVF	__pcstackBANK0, W
	MOVWF	dividend
	MOVF	al, W
	MOVWF	LCD_Char
	CALL	__awmod
	MOVF	__pcstackCOMMON, W	;10 in W
	MOVLB	0x0
	MOVWF	x
	MOVF	n, W
	MOVWF	0x5E
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	number, W
	MOVWF	0x5F
	CLRF	0x60
	MOVF	0x5F, W
	MOVWF	dividend
	MOVF	0x60, W
	MOVWF	LCD_Char
	CALL	__awdiv
	MOVF	__pcstackCOMMON, W
	MOVLB	0x0
	MOVWF	0x61
	MOVLW	0x3
A69A:
	LSLF	0x61, F
	ADDLW	0xFF
	BTFSS	STATUS, 0x2
	GOTO	A69A
	LSLF	0x61, W
	ADDWF	x, W
	RETURN

;*******************************************************************************	
;         // make editing parameter blinks function
;         void blink()
; while( j < 100 && (button1 || i >= 5) && button2 && (button3 || i < 5) )
;*******************************************************************************
	
blink:
	CLRF n			; uint8_t j = 0;
AD96:
	MOVLW	100		; wenn n kleiner 100 ist C=0
	SUBWF	n, W
	BTFSC	STATUS, 0x0
	RETURN
	
	MOVLB	0x0
	BTFSC	PORTB, 0x1	; wenn button1 bettigt ist RB1=0
	GOTO	ADA1		; RB1=1? dann zu button2
	MOVLW	0x5
	SUBWF	i, W
	BTFSS	STATUS, 0x0
	RETURN
	
ADA1:	
	BTFSS	PORTB, 0x2
	RETURN
		
	BTFSC	PORTB, 0x3
	GOTO	ADA9
	MOVLW	0x5
	SUBWF	i, W
	BTFSC	STATUS, 0x0
	RETURN
	
	GOTO	AD96
ADA9:	
	MOVLW	0x1	;j++;
	ADDWF	n, F
	Call	wait5ms   ;__delay_ms(5);
	GOTO	AD96
	
;*******************************************************************************	
;         void RTC_read()    // read current time & date from RTC chip
;*******************************************************************************
RTC_read:
           	
	CALL	I2C_Start	; I2C_Start() start I2C		
	MOVLW	0xD0	; I2C_Write(0xD0);  RTC chip address
	CALL	I2C_Write
	MOVLW	0x0	; I2C_Write(0); send register address
	CALL	I2C_Write
	CALL	0x52D	; I2C_Repeated_Start(); restart I2C
	MOVLW	0xD1	; I2C_Write(0xD1); initialize data read
	CALL	I2C_Write
	MOVLW	0x1	; second = I2C_Read(1); read seconds from register 0
	CALL	I2C_Read
	MOVWF	LCD_Char
	MOVF	LCD_Char, W
	MOVLB	0x0
	MOVWF	second
	MOVLW	0x1	; minute = I2C_Read(1); read minutes from register 1
	CALL	I2C_Read
	MOVWF	LCD_Char
	MOVF	LCD_Char, W
	MOVLB	0x0
	MOVWF	minute
	MOVLW	0x1	; hour   = I2C_Read(1); read hour from register 2
	CALL	I2C_Read
	MOVWF	LCD_Char
	MOVF	LCD_Char, W
	MOVLB	0x0
	MOVWF	hour
	MOVLW	0x1	 ; w_day  = I2C_Read(1); read day of the week from register 3
	CALL	I2C_Read
	MOVWF	LCD_Char
	MOVF	LCD_Char, W
	MOVLB	0x0
	MOVWF	w_day
	MOVLW	0x1	; m_day  = I2C_Read(1); read date from register 4
	CALL	I2C_Read
	MOVWF	LCD_Char
	MOVF	LCD_Char, W
	MOVLB	0x0
	MOVWF	m_day
	MOVLW	0x1	; month  = I2C_Read(1); read month from register 5
	CALL	I2C_Read
	MOVWF	LCD_Char
	MOVF	LCD_Char, W
	MOVLB	0x0
	MOVWF	month
	MOVLW	0x0	; year   = I2C_Read(0); read year from register 6
	CALL	I2C_Read
	MOVWF	LCD_Char
	MOVF	LCD_Char, W
	MOVLB	0x0
	MOVWF	year
	CALL	I2C_Stop	; I2C_Stop(); stop I2C
	RETURN 
	
;*******************************************************************************	
; void alarms_read_display()  read and display alarm1 and alarm2 data function
;           {
;            uint8_t control_reg, temperature_lsb;
;            int8_t temperature_msb;
;*******************************************************************************
	
alarms_read_display:

	CALL	I2C_Start	; I2C_Start();start I2C
	MOVLW	0xD0	; I2C_Write(0xD0); RTC chip address
	CALL	I2C_Write
	MOVLW	0x8	; I2C_Write(0x08); send register address (alarm1 minutes register)
	CALL	I2C_Write
	CALL	I2C_Repeated_Start	; I2C_Repeated_Start(); restart I2C
	MOVLW	0xD1	; I2C_Write(0xD1); initialize data read
	CALL	I2C_Write
	MOVLW	0x1	; alarm1_minute = I2C_Read(1); ead alarm1 minutes
	CALL	I2C_Read
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	alarm1_minute
	MOVLW	0x1	; alarm1_hour   = I2C_Read(1); read alarm1 hours
	CALL	I2C_Read
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	alarm1_hour
	MOVLW	0x1	; I2C_Read(1); skip alarm1 day/date register
	CALL	I2C_Read
	MOVLW	0x1	; alarm2_minute = I2C_Read(1); read alarm2 minutes
	CALL	I2C_Read
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	alarm2_minute
	MOVLW	0x1	; alarm2_hour   = I2C_Read(1); read alarm2 hours
	CALL	I2C_Read
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	alarm2_hour
	MOVLW	0x1	; I2C_Read(1); skip alarm2 day/date register
	CALL	I2C_Read
	MOVLW	0x1	; control_reg = I2C_Read(1); read control register
	CALL	I2C_Read
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	__pcstackBANK0
	MOVLW	0x1	; status_reg  = I2C_Read(1); read status register
	CALL	I2C_Read
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	status_reg
	MOVLW	0x1	; I2C_Read(1); skip aging offset register
	CALL	I2C_Read
	MOVLW	0x1	; temperature_msb = I2C_Read(1); read temperature MSB
	CALL	I2C_Read
	MOVF	number, W
	MOVLB	0x0
	MOVWF	al
	MOVLW	0x0	; temperature_lsb = I2C_Read(0); read temperature LSB
	CALL	I2C_Read
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	x
	CALL	I2C_Stop	; I2C_Stop(); stop I2C
; convert BCD to decimal
; alarm1_minute = bcd_to_decimal(alarm1_minute);
	MOVLB	0x0
	MOVF	alarm1_minute, W
	CALL	bcd_to_decimal
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	alarm1_minute
; alarm1_hour   = bcd_to_decimal(alarm1_hour);
	MOVF	alarm1_hour, W
	CALL	bcd_to_decimal
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	alarm1_hour
	; alarm2_minute = bcd_to_decimal(alarm2_minute);
	MOVF	alarm2_minute, W
	CALL	bcd_to_decimal
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	alarm2_minute
	; alarm2_hour   = bcd_to_decimal(alarm2_hour);
	MOVF	alarm2_hour, W
	CALL	bcd_to_decimal
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	alarm2_hour
	; end conversion
	; update Alarm1 and Alarm2
	; Alarm1[8]     = alarm1_minute % 10  + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVF	alarm1_minute, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awmod
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x0
	MOVWF	0x28
; Alarm1[7]     = alarm1_minute / 10  + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	alarm1_minute, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awdiv
	MOVF	__pcstackCOMMON, W
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x0
	MOVWF	0x27
; Alarm1[5]     = alarm1_hour   % 10  + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	alarm1_hour, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awmod
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x0
	MOVWF	0x25
; Alarm1[4]     = alarm1_hour   / 10  + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	alarm1_hour, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awdiv
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x0
	MOVWF	0x24
; Alarm2[8]     = alarm2_minute % 10  + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	alarm2_minute, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awmod
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x1
	MOVWF	0xA8
; Alarm2[7]     = alarm2_minute / 10  + '0';
	MOVLW	0xA
	MOVWF	0xF0
	MOVLW	0x0
	MOVWF	0xF1
	MOVLB	0x0
	MOVF	alarm2_minute, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awdiv
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x1
	MOVWF	0xA7
; Alarm2[5]     = alarm2_hour   % 10  + '0';
	MOVLW	0xA
	MOVWF	0xF0
	MOVLW	0x0
	MOVWF	0xF1
	MOVLB	0x0
	MOVF	alarm2_hour, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awmod
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x1
	MOVWF	0xA5
; Alarm2[4]     = alarm2_hour   / 10  + '0';
	MOVLW	0xA
	MOVWF	0xF0
	MOVLW	0x0
	MOVWF	0xF1
	MOVLB	0x0
	MOVF	alarm2_hour, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awdiv
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x1
	MOVWF	0xA4
; alarm1_status = control_reg & 0x01;      // read alarm1 interrupt enable bit (A1IE) from RTC chip control register
	MOVLB	0x0
	MOVF	__pcstackBANK0, W
	ANDLW	0x1
	MOVWF	number
	MOVF	number, W
	MOVWF	alarm1_status
; alarm2_status = (control_reg >> 1) & 0x01; // read alarm2 interrupt enable bit (A2IE) from RTC chip control register
	LSRF	__pcstackBANK0, W
	ANDLW	0x1
	MOVWF	number
	MOVF	number, W
	MOVWF	alarm2_status
; if(temperature_msb < 0){
	BTFSS	al, 0x7
	GOTO	0x3C2
; temperature_msb *= -1;
	MOVF	al, W
	MOVWF	number
	MOVF	number, W
	MOVWF	__pcstackCOMMON
	MOVLW	0xFF
	CALL	0x560
	MOVWF	0x7A
	MOVF	0x7A, W
	MOVLB	0x0
	MOVWF	al
; Temperature[2] = '-';
	MOVLW	0x2D
	GOTO	A3C3
; }
; else
; Temperature[2] = ' ';
	MOVLW	0x20
A3C3:  
	MOVWF	number
	MOVF	number, W
	MOVWF	0x2F
; temperature_lsb >>= 6;
	SWAPF	x, F
	RRF	x, F
	RRF	x, F
	MOVLW	0x3
	ANDWF	x, F
; Temperature[4] = temperature_msb % 10  + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	al, W
	MOVWF	dividend
	CLRF	LCD_Char
	BTFSC	dividend, 0x7
	DECF	LCD_Char, F
	CALL	__awmod
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	0x31
; Temperature[3] = temperature_msb / 10  + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	al, W
	MOVWF	dividend
	CLRF	LCD_Char
	BTFSC	dividend, 0x7
	DECF	LCD_Char, F
	CALL	__awdiv
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	0x30
; if(temperature_lsb == 0 || temperature_lsb == 2){
	MOVF	x, W
	BTFSC	STATUS, 0x2
	GOTO	A3F6
	MOVLW	0x2
	XORWF	x, W
	BTFSS	STATUS, 0x2
	GOTO	0x403
; Temperature[7] = '0';
A3F6: 
	MOVLW	0x30
	MOVWF	number
	MOVF	number, W
	MOVWF	0x34
; if(temperature_lsb == 0) Temperature[6] = '0';
	MOVF	x, W
	BTFSS	STATUS, 0x2
	GOTO	A3FF
	MOVLW	0x30
	GOTO	A400
; else                     Temperature[6] = '5';
A3FF: 
	MOVLW	0x35
A400: 
	MOVWF	number
	MOVF	number, W
	MOVWF	0x33
; }
; if(temperature_lsb == 1 || temperature_lsb == 3){
	DECF	x, W
	BTFSC	STATUS, 0x2
	GOTO	A40A
	MOVLW	0x3
	XORWF	x, W
	BTFSS	STATUS, 0x2
	GOTO	A417
; Temperature[7] = '5';
A40A: 
	MOVLW	0x35
	MOVWF	number
	MOVF	number, W
	MOVWF	0x34
; if(temperature_lsb == 1) Temperature[6] = '2';
	DECF	x, W
	BTFSS	STATUS, 0x2
	GOTO	A413
	MOVLW	0x32
	GOTO	A414
; else                     Temperature[6] = '7';
A413:
	MOVLW	0x37
A414:
	MOVWF	number
	MOVF	number, W
	MOVWF	0x33
; }
; Temperature[8]  = 223;   // put degree symbol (°)
A417:	
	MOVLW	0xDF
	MOVWF	number
	MOVF	number, W
	MOVWF	0x35
; LCD_Goto(11, 1);
	CLRF	row
	INCF	row, F
	MOVLW	0xB
	CALL	LCD_Goto
; LCD_Print(Temperature);  // print temperature
	MOVLW	0x2D
	MOVWF	dividend
	MOVLW	0x0
	MOVWF	LCD_Char
	CALL	LCD_Print

; LCD_Goto(1, 3);
	MOVLW	0x3
	MOVWF	number
	MOVF	number, W
	MOVWF	row
	MOVLW	0x1
	CALL	LCD_Goto

; LCD_Print(Alarm1); print Alarm1
	MOVLW	0x20
	MOVWF	dividend
	MOVLW	0x0
	MOVWF	LCD_Char
	CALL	LCD_Print

; if(alarm1_status) {
	MOVLB	0x0
	MOVF	alarm1_status, W
	BTFSC	STATUS, 0x2
	GOTO	A447
; LCD_Goto(18, 3);
	MOVLW	0x3
	MOVWF	number
	MOVF	number, W
	MOVWF	row
	MOVLW	0x12
	CALL	LCD_Goto

; LCD_Print("ON "); if A1IE = 1 print 'ON'
	MOVLW	0x4
	MOVWF	dividend
	MOVLW	0x90
	GOTO	A452
; }
; else {
; LCD_Goto(18, 3);
A447:
	MOVLW	0x3	
	MOVWF	number
	MOVF	number, W
	MOVWF	row
	MOVLW	0x12
	CALL	LCD_Goto

; LCD_Print("OFF");   // if A1IE = 0 print 'OFF'
	MOVLW	0x8
	MOVWF	dividend
	MOVLW	0x90
A452:
	MOVWF	LCD_Char
	CALL	LCD_Print
; }
; LCD_Goto(1, 4);
	MOVLW	0x4
	MOVWF	number
	MOVF	number, W
	MOVWF	row
	MOVLW	0x1
	CALL	LCD_Goto

; LCD_Print(Alarm2);    // print Alarm2
	MOVLW	0xA0
	MOVWF	dividend
	MOVLW	0x0
	MOVWF	LCD_Char
	CALL	LCD_Print

; if(alarm2_status){
	MOVLB	0x0
	MOVF	alarm2_status, W
	BTFSC	STATUS, 0x2
	GOTO	A478
; LCD_Goto(18, 4);
	MOVLW	0x4
	MOVWF	number
	MOVF	number, W
	MOVWF	row
	MOVLW	0x12
	CALL	LCD_Goto

; LCD_Print("ON ");   // if A2IE = 1 print 'ON'
	MOVLW	0x4
	MOVWF	dividend
	MOVLW	0x90
	MOVWF	LCD_Char
	CALL	LCD_Print
	RETURN
; else {
; LCD_Goto(18, 4);
A478:
	MOVLW	0x4
	MOVWF	number
	MOVF	number, W
	MOVWF	row
	MOVLW	0x12
	CALL	LCD_Goto

; LCD_Print("OFF");   // if A2IE = 0 print 'OFF'
	MOVLW	0x8
	MOVWF	dividend
	MOVLW	0x90
	MOVWF	LCD_Char
	CALL	LCD_Print
	RETURN

;*******************************************************************************	
;		void w_day_display() print day of the week
;{
;             LCD_Goto(1, 2);
;*******************************************************************************	

w_day_display:
	
	MOVLW	0x2
	MOVWF	0x78
	MOVF	0x78, W
	MOVWF	row
	MOVLW	0x1
	CALL	LCD_Goto
; switch(w_day){
	MOVLB	0x0
	MOVF	w_day, W
	XORLW	0x1
	BTFSC	STATUS, 0x2
	GOTO	AFB9
	XORLW	0x3
	BTFSC	STATUS, 0x2
	GOTO	AFC0
	XORLW	0x1
	BTFSC	STATUS, 0x2
	GOTO	AFC7
	XORLW	0x7
	BTFSC	STATUS, 0x2
	GOTO	AFCE
	XORLW	0x1
	BTFSC	STATUS, 0x2
	GOTO	AFD5
	XORLW	0x3
	BTFSC	STATUS, 0x2
	GOTO	AFDC
	GOTO	AFE3
; case 1:  LCD_Print("Sun"); break;
AFB9:	
	MOVLW	0x1C
	MOVWF	dividend
	MOVLW	0x90
	MOVWF	LCD_Char
	CALL	LCD_Print
	RETURN
; case 2:  LCD_Print("Mon"); break;
AFC0:	
	MOVLW	0x18
	MOVWF	dividend
	MOVLW	0x90
	MOVWF	LCD_Char
	CALL	LCD_Print
	RETURN
; case 3:  LCD_Print("Tue"); break;
AFC7:	
	MOVLW	0x10
	MOVWF	dividend
	MOVLW	0x90
	MOVWF	LCD_Char
	CALL	LCD_Print
	RETURN
; case 4:  LCD_Print("Wed"); break;
AFCE:
	MOVLW	0xC
	MOVWF	dividend
	MOVLW	0x90
	MOVWF	LCD_Char
	CALL	LCD_Print
	RETURN
; case 5:  LCD_Print("Thu"); break;
AFD5:
	MOVLW	0x24
	MOVWF	dividend
	MOVLW	0x90
	MOVWF	LCD_Char
	CALL	LCD_Print
	RETURN
; case 6:  LCD_Print("Fri"); break;
AFDC:	
	MOVLW	0x14
	MOVWF	dividend
	MOVLW	0x90
	MOVWF	LCD_Char
	CALL	LCD_Print
	RETURN
; default: LCD_Print("Sat");
AFE3:
	MOVLW	0x20
	MOVWF	dividend
	MOVLW	0x90
	MOVWF	LCD_Char
	CALL	LCD_Print            
	RETURN

;*******************************************************************************	
;          // edit time and date function
;           uint8_t edit(uint8_t x, uint8_t y, uint8_t parameter)
;*******************************************************************************
	
edit:	
	MOVLB	0x00
	MOVWF	x		
A5C9:	;while(debounce(1) || debounce(3));Schleife solange Button 1 oder Button 3 bettigt
	MOVLW	0x1
	CALL	debounce
	BTFSC	STATUS, 0x0
	GOTO	A5C9
	MOVLW	0x3
	CALL	debounce
	BTFSS	STATUS, 0x0
	GOTO	TestButt2
	GOTO	A5C9
	
TestButt2:
	MOVLB	0x0	; while(!button2) if button B2 is pressed
	BTFSS	PORTB, 0x2
	Goto	LCD_Goto		
ParaHours:
	MOVLW	1		; parameter++;
	ADDWF	parameter, F	; al = parameter
	MOVF	i, W		; if(((i == 0) || (i == 5)) && parameter > 23)
	BTFSC	STATUS, 2	; if hours > 23 ==> hours = 0
	GOTO	SetHours
	MOVLW	5
	XORWF	i, W
	BTFSS	STATUS, 2
	GOTO	ParaMinutes
SetHours:
	MOVLW	24
	SUBWF	parameter, W
	BTFSC	STATUS, 0               
	CLRF	parameter	; parameter = 0;
ParaMinutes:
	DECF	i, W		; if(((i == 1) || (i == 6)) && parameter > 59)  
	BTFSC	STATUS, 0x2	; if minutes > 59 ==> minutes = 0
	GOTO	SetMinutes
	MOVLW	6
	XORWF	i, W
	BTFSS	STATUS, 0x2
	GOTO	ParaDate
SetMinutes:	
	MOVLW	60
	SUBWF	parameter, W
	BTFSC	STATUS, 0             
	CLRF	parameter	; parameter = 0;
ParaDate:
	MOVLW	2		; if(i == 2 && parameter > 31)
	XORWF	i, W		; if date > 31 ==> date = 1
	BTFSS	STATUS, 2
	GOTO	ParaMonth
	MOVLW	32
	SUBWF	parameter, W
	BTFSS	STATUS, 0
	GOTO	ParaMonth			
	CLRF	parameter	; parameter = 1;
	INCF	parameter, F	
ParaMonth:
	MOVLW	3		; if(i == 3 && parameter > 12)   
	XORWF	i, W		; if month > 12 ==> month = 1
	BTFSS	STATUS, 2
	GOTO	ParaYear
	MOVLW	13
	SUBWF	parameter, W
	BTFSS	STATUS, 0
	GOTO	ParaYear
	CLRF	parameter	; parameter = 1;
	INCF	parameter, F
ParaYear:
	MOVLW	4		; if(i == 4 && parameter > 99)
	XORWF	i, W		; if year > 99 ==> year = 0
	BTFSS	STATUS, 2
	GOTO	ParaAlarms
	MOVLW	0x64
	SUBWF	parameter, W
	BTFSS	STATUS, 0
	GOTO	ParaAlarms
	CLRF	parameter	; parameter = 0;
ParaAlarms:
	MOVLW	7		; if(i == 7 && parameter > 1)
	XORWF	i, W		; for alarms ON or OFF (1: alarm ON, 0: alarm OFF)
	BTFSS	STATUS, 2
	GOTO	LcdGotoEdit
	MOVLW	0x2
	SUBWF	parameter, W
	BTFSS	STATUS, 0
	GOTO	LcdGotoEdit
	CLRF	parameter	; parameter = 0;
	
LcdGotoEdit:          
; LCD_Goto(x, y);
	MOVF	__pcstackBANK0, W
	MOVWF	0x7C
	MOVF	0x7C, W
	MOVWF	row
	MOVF	x, W
	CALL	LCD_Goto
; LCD_PutC(parameter / 10 + '0');
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	button
	MOVLB	0x0
	MOVF	al, W
	MOVWF	0x7C
	CLRF	counter
	MOVF	0x7C, W
	MOVWF	count
	MOVF	counter, W
	MOVWF	i
	CALL	__awdiv
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	CALL	LCD_PutC
; LCD_PutC(parameter % 10 + '0');
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	button
	MOVLB	0x0
	MOVF	al, W
	MOVWF	0x7C
	CLRF	counter
	MOVF	0x7C, W
	MOVWF	count
	MOVF	counter, W
	MOVWF	i
	CALL	__awmod
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	CALL	LCD_PutC         
	MOVLW	0x5		; if(i >= 5){
	MOVLB	0x0
	SUBWF	i, W
	BTFSS	STATUS, 0x0
	GOTO	A65F
	CALL	RTC_read	; read data from RTC chip
	CALL	RTC_display	; display time and date
A65F:	
	CALL	wait5ms		;__delay_ms(200);
; LCD_Goto(x, y);
	MOVF	__pcstackBANK0, W
	MOVWF	0x7C
	MOVF	0x7C, W
	MOVWF	row
	MOVF	x, W
	CALL	LCD_Goto
; LCD_Print("  ");  print 2 spaces
	MOVLW	0x1
	MOVWF	count
	MOVLW	0x90
	MOVWF	i
	CALL	LCD_Print
	CALL	blink 
; LCD_Goto(x, y);
	MOVLB	0x0
	MOVF	__pcstackBANK0, W
	MOVWF	0x7C
	MOVF	0x7C, W
	MOVWF	row
	MOVF	x, W
	CALL	LCD_Goto
; LCD_PutC(parameter / 10 + '0');
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	button
	MOVLB	0x0
	MOVF	al, W
	MOVWF	0x7C
	CLRF	counter
	MOVF	0x7C, W
	MOVWF	count
	MOVF	counter, W
	MOVWF	i
	MOVLP	0xF
	CALL	__awdiv
	MOVLP	0x5
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	CALL	LCD_PutC

; LCD_PutC(parameter % 10 + '0');
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	button
	MOVLB	0x0
	MOVF	al, W
	MOVWF	0x7C
	CLRF	counter
	MOVF	0x7C, W
	MOVWF	count
	MOVF	counter, W
	MOVWF	i
	CALL	__awmod
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	CALL	LCD_PutC
	CALL	blink    
; if(i >= 5)
A6B4:
	MOVLW	0x5
	MOVLB	0x0
	SUBWF	i, W
	BTFSS	STATUS, 0x0
	GOTO	A6BF
	CALL	RTC_read
	CALL	RTC_display
; if((!button1 && i < 5) || (!button3 && i >= 5)) if B1 or B3 is pressed
A6BF:
	MOVLB	0x0
A6C0:
	BTFSC	PORTB, 0x1
	GOTO	RTC1
	MOVLW	0x5
	SUBWF	i, W
	BTFSS	STATUS, 0x0
	GOTO	A6CC
RTC1:
	BTFSC	PORTB, 0x3
	GOTO	TestButt2
	MOVLW	0x5
	SUBWF	i, W
	BTFSS	STATUS, 0x0
	GOTO	TestButt2
A6CC:	MOVLW	0x1	;i++; increment 'i' for the next parameter
	MOVWF	0x7C
	MOVF	0x7C, W
	ADDWF	i, F
	MOVF	al, W	; return parameter; return parameter value and exit
	RETURN
	
	
;          // display time and date function
;           void RTC_display()
;       // convert data from BCD format to decimal format
	
	
RTC_display:	
; second = bcd_to_decimal(second);
	MOVLB	0x0
	MOVF	second, W
	CALL	bcd_to_decimal
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	second
; minute = bcd_to_decimal(minute);
	MOVF	minute, W
	CALL	bcd_to_decimal
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	minute
; hour   = bcd_to_decimal(hour);
	MOVF	hour, W
	CALL	bcd_to_decimal
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	hour
; m_day  = bcd_to_decimal(m_day);
	MOVF	m_day, W
	CALL	bcd_to_decimal
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	m_day
; month  = bcd_to_decimal(month);
	MOVF	month, W
	CALL	bcd_to_decimal
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	month
; year   = bcd_to_decimal(year);
	MOVF	year, W
	CALL	bcd_to_decimal
	MOVWF	number
	MOVF	number, W
	MOVLB	0x0
	MOVWF	year
; end conversion
; update time
; Time[7] = second % 10  + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	second, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awmod
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x0
	MOVWF	0x4A
; Time[6] = second / 10  + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	second, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awdiv
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x0
	MOVWF	0x49
; Time[4] = minute % 10  + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	minute, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awmod
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x0
	MOVWF	0x47
; Time[3] = minute / 10  + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	minute, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awdiv
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x0
	MOVWF	0x46
; Time[1] = hour   % 10  + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	hour, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awmod
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x0
	MOVWF	0x44
; Time[0] = hour   / 10  + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	hour, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awdiv
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x0
	MOVWF	Time
;  update date
; Date[9] = year  % 10 + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	year, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awmod
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x0
	MOVWF	0x41
; Date[8] = year  / 10 + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	year, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awdiv
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x0
	MOVWF	0x40
; Date[4] = month % 10 + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	month, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awmod
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x0
	MOVWF	0x3C
; Date[3] = month / 10 + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	month, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awdiv
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x0
	MOVWF	0x3B
; Date[1] = m_day  % 10 + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	m_day, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awmod
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x0
	MOVWF	0x39
; Date[0] = m_day  / 10 + '0';
	MOVLW	0xA
	MOVWF	__pcstackCOMMON
	MOVLW	0x0
	MOVWF	n
	MOVF	m_day, W
	MOVWF	number
	CLRF	0x7A
	MOVF	number, W
	MOVWF	dividend
	MOVF	0x7A, W
	MOVWF	LCD_Char
	CALL	__awdiv
	MOVF	__pcstackCOMMON, W
	ADDLW	0x30
	MOVWF	0x7B
	MOVF	0x7B, W
	MOVLB	0x0
	MOVWF	Date        
; LCD_Goto(1, 1);    // go to column 1, row 1
	CLRF	row
	INCF	row, F
	MOVLW	0x1
	CALL	LCD_Goto
; LCD_Print(Time);   // print time
	MOVLW	0x43
	MOVWF	dividend
	MOVLW	0x0
	MOVWF	LCD_Char
	CALL	LCD_Print
; LCD_Goto(5, 2);    // go to column 1, row 2
	MOVLW	0x2
	MOVWF	number
	MOVF	number, W
	MOVWF	row
	MOVLW	0x5
	CALL	LCD_Goto
; LCD_Print(Date);   // print date
	MOVLW	0x38
	MOVWF	dividend
	MOVLW	0x0
	MOVWF	LCD_Char
	CALL	LCD_Print
; w_day_display();   // print day of the week
	CALL	w_day_display
	RETURN
	
;*******************************************************************************
;          uint8_t on_off(uint8_t x, uint8_t y, uint8_t al)
;*******************************************************************************
	
on_off:	
	MOVLB	0x0
	MOVWF	x
A6C6:	; while(debounce(3)); call debounce function (wait for B3 to be released)
	MOVLW	0x3
	CALL	debounce
	BTFSS	STATUS, 0x0
	GOTO	A6FA
	GOTO	A6C6
A6FA:	; while(!button2) if button B2 is pressed
	MOVLB	0x0
	BTFSS	PORTB, 0x2
	GOTO	A6CD
A6CD:	; al = (!al) & 0x01;
	MOVF	al, W
	BTFSS	STATUS, 0x2
	GOTO	A6D2
	MOVLW	0x1
	GOTO	A6D3
A6D2:
	MOVLW	0x0
A6D3:
	ANDLW	0x1
	MOVWF	0x7C
	MOVF	0x7C, W
	MOVWF	al
	MOVF	__pcstackBANK0, W	; LCD_Goto(x, y);
	MOVWF	0x7C
	MOVF	0x7C, W
	MOVWF	row
	MOVF	x, W
	CALL	LCD_Goto
	MOVLB	0x0	; if(al == 1)  LCD_Print("ON ");
	DECF	al, W
	BTFSS	STATUS, 0x2
	GOTO	A6E7
	MOVLW	0x4
	MOVWF	dividend
	MOVLW	0x90
	GOTO	A6EA
A6E7:	; else LCD_Print("OFF");
	MOVLW	0x8
	MOVWF	dividend
	MOVLW	0x90
A6EA:
	MOVWF	LCD_Char
	CALL	LCD_Print
	CALL	wait5ms   ;__delay_ms(200);
	MOVF	__pcstackBANK0, W; LCD_Goto(x, y);
	MOVWF	0x7C
	MOVF	0x7C, W
	MOVWF	row
	MOVF	x, W
	CALL	LCD_Goto
	MOVLW	0x0	; LCD_Print("   "); print 3 spaces
	MOVWF	dividend
	MOVLW	0x90
	MOVWF	LCD_Char
	CALL	LCD_Print
	CALL	blink	; blink();          
	MOVLB	0x0	; LCD_Goto(x, y);
	MOVF	__pcstackBANK0, W
	MOVWF	0x7C
	MOVF	0x7C, W
	MOVWF	row
	MOVF	x, W
	CALL	LCD_Goto
	MOVLB	0x0	; if(al == 1)  LCD_Print("ON ");
	DECF	al, W
	BTFSS	STATUS, 0x2
	GOTO	A720
	MOVLW	0x4
	MOVWF	dividend
	MOVLW	0x90
	GOTO	A723
A720:	; else LCD_Print("OFF");
	MOVLW	0x8
	MOVWF	dividend
	MOVLW	0x90
A723:
	MOVWF	LCD_Char
	CALL	LCD_Print
	CALL	blink	; blink();
	CALL	RTC_read	; RTC_read();
	CALL	RTC_display	; RTC_display();   
	MOVLB	0x0	; if(!button3)
	BTFSC	PORTB, 0x3
	GOTO	A6FA
	MOVLW	0x3	; if(debounce(3))
	CALL	debounce
	BTFSS	STATUS, 0x0
	GOTO	A6FA
	MOVLB	0x0	; return al; 
	MOVF	al, W
	RETURN
;*******************************************************************************	
;++++++++++++++++++++++++++  I2C-Funktions   +++++++++++++++++++++++++++++++++++
;*******************************************************************************
	
;*******************************************************************************
;            void I2C_Init(uint32_t i2c_clk_freq)
;*******************************************************************************
	
I2C_Init:
	MOVLW	0x28	; SSP1CON1  = 0x28; configure MSSP module to work in I2C mode
	MOVLB	0x3
	MOVWF	SSP1CON1
	MOVLW	0x4F	; SSP1ADD  = 0x4F; set I2C clock frequency
	MOVWF	SSP1ADD	; (_XTAL_FREQ /(4 * i2c_clk_freq))-1 = (32000000Mhz/(4*100000khz) -1 = 79 = 0x4F
	CLRF	SSP1STAT; SSP1STAT = 0;
	RETURN
	
;*******************************************************************************             
;           void I2C_Start()
;******************************************************************************* 
	
I2C_Start:	
	MOVLB	0x3		; while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));
	BTFSC	SSP1STAT, 0x2	; wait for MSSP module to be free (not busy)
	GOTO	I2C_Start
	MOVF	SSP1CON2, W
	ANDLW	0x1F
	BTFSS	STATUS, 0x2
	GOTO	I2C_Start
	BSF	SSP1CON2, 0x0	; SEN1 = 1;  // initiate start condition
	RETURN
;******************************************************************************* 
;       void I2C_Repeated_Start()
;******************************************************************************* 
  
I2C_Repeated_Start:
	
ACA4:	
	MOVLB	0x3		; while((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));
	BTFSC	SSP1STAT, 0x2	; wait for MSSP module to be free (not busy)
	GOTO	ACA4
	MOVF	SSP1CON2, W
	ANDLW	0x1F
	BTFSS	STATUS, 0x2
	GOTO	ACA4
	BSF	SSP1CON2, 0x1	; RSEN1 = 1; initiate repeated start condition
	RETURN
;*******************************************************************************          
;				void I2C_Stop()
;*******************************************************************************
	
I2C_Stop:
  
ACB6:	
	MOVLB	0x3		; while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));
	BTFSC	SSP1STAT, 0x2	; wait for MSSP module to be free (not busy)
	GOTO	ACB6
	MOVF	SSP1CON2, W
	ANDLW	0x1F
	BTFSS	STATUS, 0x2
	GOTO	ACB6
	BSF	SSP1CON2, 0x2	; PEN1 = 1;  // initiate stop condition
	RETURN
;*******************************************************************************           
;			 void I2C_Write(uint8_t i2c_data)
;*******************************************************************************
I2C_Write:
	MOVWF	0x1F0
ACCB:	
	MOVLB	0x3		; while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F)); 
	BTFSC	SSP1STAT, 0x2	; wait for MSSP module to be free (not busy)
	GOTO	ACCB
	MOVF	SSP1CON2, W
	ANDLW	0x1F
	BTFSS	STATUS, 0x2
	GOTO	ACCB
	MOVF	0x1F0, W
	MOVWF	SSP1BUF		; SSP1BUF = i2c_data;  // update buffer
	RETURN
	
;*******************************************************************************           
;			uint8_t I2C_Read(uint8_t ack)
;*******************************************************************************           

I2C_Read:
	MOVWF	0x1F1
AD70:	
	MOVLB	0x3		; while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));
	BTFSC	SSP1STAT, 0x2	; wait for MSSP module to be free (not busy)
	GOTO	AD70
	MOVF	SSP1CON2, W
	ANDLW	0x1F
	BTFSS	STATUS, 0x2
	GOTO	AD70
	BSF	SSP1CON2, 0x3	; RCEN1 = 1;
; while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F))
AD78:	;wait for MSSP module to be free (not busy)
	BTFSC	SSP1STAT, 0x2
	GOTO	AD78
	MOVF	SSP1CON2, W
	ANDLW	0x1F
	BTFSS	STATUS, 0x2
	GOTO	AD78
; _data = SSP1BUF; read data from buffer
	MOVF	SSP1BUF, W
	MOVWF	0x1F0
	MOVF	0x1F0, W
	MOVWF	0x1F2
; while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F))
AD82:	; wait for MSSP module to be free (not busy)
	BTFSC	SSP1STAT, 0x2
	GOTO	AD82
	MOVF	SSP1CON2, W
	ANDLW	0x1F
	BTFSS	STATUS, 0x2
	GOTO	AD82
	MOVF	0x1F1, W; send acknowledge pulse ? (depends on ack, if 1 send, otherwise don't send)
	BTFSS	STATUS, 0x2	; ACKDT1 = !ack;
	GOTO	AD8E
	MOVLB	0x3
	BSF	SSP1CON2, 0x5
	GOTO	AD90
AD8E:	
	MOVLB	0x3
	BCF	SSP1CON2, 0x5
AD90:
	BSF	SSP1CON2, 0x4	; ACKEN1 = 1;
	MOVF	0x1F2, W	; return _data; return data read
	RETURN
;*******************************************************************************
;			 function for button debounce
;			 __bit debounce (uint8_t button)
;*******************************************************************************	
debounce: 
	MOVWF	ButtonFlags	; Buttonflags
	CLRF	count		; uint8_t count = 0;
	CLRF	i		; for(uint8_t i = 0; i < 5; i++)
	GOTO	AE3B
AE33:
	INCF	i, F
	MOVLW	0x5
	SUBWF	i, W
	BTFSC	STATUS, 0x0
	GOTO	AE44
	GOTO	AE2B
AE3B:
	MOVF	ButtonFlags, W	; switch (button)
	XORLW	0x1
	BTFSC	STATUS, 0x2
	GOTO	AE20
	XORLW	0x2
	BTFSC	STATUS, 0x2
	GOTO	AE28
	GOTO	AE2B
AE20:
	MOVLB	0x0		; case 1: if (button1 == 0)
	BTFSC	PORTB, 0x1
	GOTO	AE2B
AE23:
	INCF	count, F	; count++;
	GOTO	AE2B
AE28:	
	MOVLB	0x0		; break;
	BTFSS	PORTB, 0x3	; case 3: if (button3 == 0)
	GOTO	AE23	
AE2B:
	CALL	wait245ms	;__delay_ms(10);
	GOTO	AE33
AE44:
	MOVLW	0x3		; if(count > 2)  return 1;
	SUBWF	count, W	; else return 0;
	RETURN
;*******************************************************************************	
;++++++++++++++++++++++++++  Math-Funktions  ++++++++++++++++++++++++++++++++++
;*******************************************************************************	

;*******************************************************************************	
;		__bmul(unsigned char multiplier, unsigned char multiplicand)
;*******************************************************************************
	
__bmul:
	MOVWF	multiplier
	CLRF	product			;unsigned char product = 0;
A8E:	
	BTFSS	multiplier, 0x0		; if(multiplier & 1)
	GOTO	A96
	MOVF	__pcstackCOMMON, W	; product += multiplicand;
	ADDWF	product, F
A96: 
	BCF	STATUS, 0x0		; multiplicand <<= 1;
	RLF	__pcstackCOMMON, F
	BCF	STATUS, 0x0		; multiplier >>= 1;
	RRF	multiplier, F
	MOVF	multiplier, W		;} while(multiplier != 0);
	BTFSS	STATUS, 0x2
	GOTO	A8E
	MOVF	product, W		;return product;
	RETURN

;--  /opt/microchip/xc8/v2.20/pic/sources/c90/common/awmod.c  ------------------
;             // integer signed unsigned modulus
;             
;             signed int
;             #ifdef __PICC__
;             __awmod(signed int divisor, signed int dividend)
;             #else
;             __awmod(signed int dividend, signed int divisor)
;             #endif
;             {
;            	unsigned char	counter, sign;
;            
__awmod:
	
;AF1E:
	CLRF sign		; sign = 0;
; if(dividend < 0) {
	BTFSS LCD_Char, 0x7
	GOTO AF28
; dividend = -dividend;
	COMF dividend, F
	COMF LCD_Char, F
	INCF dividend, F
	BTFSC STATUS, 0x2
	INCF LCD_Char, F
; sign = 1;
	CLRF sign
	INCF sign, F
; }
; f(divisor < 0)
AF28:
	BTFSS n, 0x7
	GOTO AF2F
; divisor = -divisor;
	COMF __pcstackCOMMON, F
	COMF n, F
	INCF __pcstackCOMMON, F
	BTFSC STATUS, 0x2
	INCF n, F
; if(divisor != 0) {
AF2F:
	MOVF __pcstackCOMMON, W
	IORWF n, W
	BTFSC STATUS, 0x2
	GOTO AF56
; counter = 1;
	CLRF counter
	INCF counter, F
; while(((unsigned int)divisor & 0x8000U) == 0) {
AF35:
	BTFSC n, 0x7
	GOTO AF41
	GOTO AF35
; divisor <<= 1;
	MOVLW 0x1
AF38:
	LSLF __pcstackCOMMON, F
	RLF n, F
	DECFSZ WREG, F
	GOTO AF38
; counter++;
	MOVLW 0x1
	MOVWF row
	MOVF row, W
	ADDWF counter, F
; }
; do {
; if((unsigned int)divisor <= (unsigned int)dividend)
AF41:
	MOVF n, W
	SUBWF LCD_Char, W
	BTFSS STATUS, 0x2
	GOTO AF47
	MOVF __pcstackCOMMON, W
	SUBWF dividend, W
	
AF47:
	BTFSS STATUS, 0x0
	GOTO AF4D
; dividend -= divisor;
	MOVF __pcstackCOMMON, W
	SUBWF dividend, F
	MOVF n, W
	SUBWFB LCD_Char, F
; *(unsigned int *)&divisor >>= 1;
AF4D:	
	MOVLW 0x1
AF4E:
	LSRF n, F
	RRF __pcstackCOMMON, F
	DECFSZ WREG, F
	GOTO AF4E
; } while(--counter != 0);
	MOVLW 0x1
	SUBWF counter, F
	BTFSS STATUS, 0x2
	GOTO AF41
; }
; if(sign)
AF56:
	MOVF sign, W
	BTFSC STATUS, 0x2
	GOTO AF5E
; dividend = -dividend;
	COMF dividend, F
	COMF LCD_Char, F
	INCF dividend, F
	BTFSC STATUS, 0x2
	INCF LCD_Char, F
; return dividend;
AF5E:	MOVF LCD_Char, W
	MOVWF n
	MOVF dividend, W
	MOVWF __pcstackCOMMON
	RETURN

;---  /opt/microchip/xc8/v2.20/pic/sources/c90/common/awdiv.c  -----------------
;             // integer signed division
;             
;             signed int
;             #ifdef __PICC__
;             __awdiv(signed int divisor, signed int dividend)
;             #else
;             __awdiv(signed int dividend, signed int divisor)
;             #endif
;             {
;            	signed int	quotient;
;            	unsigned char	counter, sign;
;            
__awdiv:	; Dividend : Divisor = Quotient
	
;A7AB:
	CLRF	sign			; sign = 0;
	BTFSS	n, 0x7			; if(divisor < 0) Note:signed int=10000000 = -128!
	GOTO	A7B5
	COMF	__pcstackCOMMON, F	; divisor = -divisor;
	COMF	n, F
	INCF	__pcstackCOMMON, F
	BTFSC	STATUS, 0x2
	INCF	n, F
	CLRF	sign	
	INCF	sign, F			; sign = 1;
A7B5:
	BTFSS	LCD_Char, 0x7		; if(dividend < 0) {
	GOTO	A7C0
	COMF	dividend, F		; dividend = -dividend;
	COMF	LCD_Char, F
	INCF	dividend, F
	BTFSC	STATUS, 0x2
	INCF	LCD_Char, F
	MOVLW	0x1			; sign ^= 1;
	MOVWF	row
	MOVF	row, W
	XORWF	sign, F
A7C0:
	CLRF	quotient		; quotient = 0;
	CLRF	0x78
	MOVF	__pcstackCOMMON, W	; if(divisor != 0) {
	IORWF	n, W
	BTFSC	STATUS, 0x2
	GOTO	A7EF
	CLRF	counter
	INCF	counter, F		; counter = 1;

A7C8:	
	BTFSC	n, 0x7			; while((divisor & 0x8000U) == 0) {
	GOTO	A7D4
	GOTO	A7C8
	MOVLW	0x1			; divisor <<= 1;
A7CB:	
	LSLF	__pcstackCOMMON, F
	RLF	n, F
	DECFSZ	WREG, F
	GOTO	A7CB
	MOVLW	0x1			; counter++;
	MOVWF	row
	MOVF	row, W
	ADDWF	counter, F
A7D4:
	MOVLW	0x1	; quotient <<= 1;
A7D5:  
	LSLF	quotient, F
	RLF	0x78, F
	DECFSZ	WREG, F
	GOTO	A7D5
; if((unsigned int)divisor <= (unsigned int)dividend) {
	MOVF	n, W
	SUBWF	LCD_Char, W
	BTFSS	STATUS, 0x2
	GOTO	A7DF
	MOVF	__pcstackCOMMON, W
	SUBWF	dividend, W
A7DF:	
	BTFSS	STATUS, 0x0
	GOTO	A7E6
	MOVF	__pcstackCOMMON, W	; dividend -= divisor;
	SUBWF	dividend, F
	MOVF	n, W
	SUBWFB	LCD_Char, F
	BSF	quotient, 0x0	; quotient |= 1;

A7E6:
	MOVLW	0x1	; *(unsigned int *)&divisor >>= 1;
A7E7:	
	LSRF	n, F
	RRF	__pcstackCOMMON, F
	DECFSZ	WREG, F
	GOTO	A7E7
	MOVLW	0x1	; } while(--counter != 0);
	SUBWF	counter, F
	BTFSS	STATUS, 0x2
	GOTO	A7D4
A7EF:	
	MOVF	sign, W	; if(sign)
	BTFSC	STATUS, 0x2
	GOTO	A7F7
	COMF	quotient, F	; quotient = -quotient;
	COMF	0x78, F
	INCF	quotient, F
	BTFSC	STATUS, 0x2
	INCF	0x78, F
A7F7:	
	MOVF	0x78, W	; return quotient;
	MOVWF	n
	MOVF	quotient, W
	MOVWF	__pcstackCOMMON
	RETURN
	
;*******************************************************************************
;			Warteroutinen verschiedener Lnge
;*******************************************************************************

; 5 ms warten
wait5ms:
	movlw	5			; 5ms
	movwf	loops	
	goto	WAIT

; 75 ms warten
wait75ms:
	movlw	75			; 75ms
	movwf	loops	
	goto	WAIT

;245 ms warten
wait245ms:
	movlw	245			; 245ms
	movwf	loops	
	goto	WAIT	

;*******************************************************************************
; Zeitverzgerung um loops * 1 ms
; 10 MHz externer Takt bedeutet 2,5 MHz interner Takt,also dauert 1 ms 
; 2500 Befehle 250 Schleifen a 10 Befehle sind 2500 Befehle = 1 ms
;*******************************************************************************
WAIT:
top:	movlw   250		; timing adjustment variable (1ms)
        movwf   loops2
top2:   nop 			; sit and wait
        nop
        nop
        nop
	nop
        nop
        nop
        decfsz  loops2, f	; innere Schleife beendet?
        goto    top2 		; nein, noch mal rum
        decfsz  loops, f	; uere Schleife beendet?
        goto    top 		; nein, noch mal rum
        return 			; ja, return von WAIT
	
;*******************************************************************************	
;++++++++++++++++++++++++++  LCD-Funktions   +++++++++++++++++++++++++++++++++++
;*******************************************************************************	
	
;*******************************************************************************	
;				void LCD_Begin() {
;*******************************************************************************
LCD_Begin:
	CLRF	PORTD	;LCD_RS,LCD_EN & LCD_D4...D7 =0
	MOVLW	11100000B
	MOVWF	TRISD	;;LCD_RS,LCD_EN & LCD_D4...D7 auf Ausgang stellen
	Call    wait245ms ;__delay_ms(40);             
	MOVLW	0x3	;LCD_Cmd(3);
	CALL	LCD_Cmd
	Call    wait245ms	;__delay_ms(5);
	MOVLW	0x3	;LCD_Cmd(3);
	CALL	LCD_Cmd
	Call    wait245ms;__delay_ms(5);
	MOVLW	0x3	;LCD_Cmd(3);
	CALL	LCD_Cmd
	Call    wait245ms;__delay_ms(5);
	MOVLW	0x3	;LCD_Cmd(3);
	CALL	LCD_Cmd               
	MOVLW	0x2;	LCD_Cmd(LCD_RETURN_HOME);
	CALL	LCD_Cmd
	Call    wait245ms	;__delay_ms(5);               
	MOVLW	0x28	;LCD_Cmd(0x20 | (LCD_TYPE << 2));
	CALL	LCD_Cmd
	Call    wait245ms;__delay_ms(50);            
	MOVLW	0xC	;LCD_Cmd(LCD_TURN_ON);
	CALL	LCD_Cmd
	Call    wait245ms;__delay_ms(50);  
	MOVLW	0x1
	CALL	LCD_Cmd
	Call    wait245ms;__delay_ms(50);
	MOVLW	0x6	;LCD_Cmd(LCD_ENTRY_MODE_SET | LCD_RETURN_HOME);
	CALL	LCD_Cmd
	Call    wait245ms;__delay_ms(50);  
	RETURN
;*******************************************************************************
;            void LCD_Cmd(uint8_t Command) {
;*******************************************************************************
LCD_Cmd:
	MOVWF	i
	BCF	PORTD,0x0		;RS = 0;             
	MOVF	i, W
	MOVWF	count
	MOVLW	0x4
AD32:	
	LSRF	count, F	;LCD_Write_Nibble(Command >> 4);
	DECFSZ	WREG, F
	GOTO	AD32
	MOVF	count, W
	CALL	LCD_Write_Nibble
	MOVF	i, W
	CALL	LCD_Write_Nibble;LCD_Write_Nibble(Command);
	DECF	i, W ; if ((Command == LCD_CLEAR)||(Command == LCD_RETURN_HOME)) 
	BTFSC	STATUS, 0x2
	GOTO	AD44
	MOVLW	0x2
	XORWF	i, W
	BTFSS	STATUS, 0x2
	RETURN
AD44:	
	Call    wait245ms	;__delay_ms(2);  
	RETURN
;*******************************************************************************	
;            void LCD_Goto(uint8_t col, uint8_t row) {
;*******************************************************************************
LCD_Goto:
	MOVWF	col               
	GOTO	AD63
AD63:	MOVF	row, W	;switch (row) {
	XORLW	0x2
	BTFSC	STATUS, 0x2
	GOTO	AD4F
	XORLW	0x1
	BTFSC	STATUS, 0x2
	GOTO	AD54
	XORLW	0x7
	BTFSC	STATUS, 0x2
	GOTO	AD59
	GOTO	AD5E
; case 2:
; LCD_Cmd(LCD_SECOND_ROW + col - 1);
AD4F:	
	MOVF	col, W
	ADDLW	0xBF
	CALL	LCD_Cmd             
	RETURN	;break;
; case 3:
; LCD_Cmd(LCD_THIRD_ROW + col - 1);
AD54:	MOVF	col, W
	ADDLW	0x93
	CALL	LCD_Cmd                    
	RETURN	;break;
; case 4:
; LCD_Cmd(LCD_FOURTH_ROW + col - 1);
AD59:	MOVF	col, W
	ADDLW	0xD3
	CALL	LCD_Cmd 
	RETURN	; break;
; default: // case 1:
; LCD_Cmd(LCD_FIRST_ROW + col - 1);
AD5E:	MOVF	col, W
	ADDLW	0x7F; schon -1 von LCD_FIRST_ROW abgezogen!?
	CALL	LCD_Cmd
	RETURN

;*******************************************************************************	
;            void LCD_Write_Nibble(uint8_t n) {
;*******************************************************************************	
	
LCD_Write_Nibble: 
	MOVWF	button           
	MOVLB	0x0	;LCD_RS = RS;
	BTFSS	PORTD, 0x0	;RD0
	GOTO	AE84
	MOVLB	0x0
	BSF	PORTD, 0x0
	GOTO	A686
AE84:
	MOVLB	0x0
	BCF	PORTD, 0x0           
A686:	BTFSS	button, 0x0 ; LCD_D4 = n & 0x01;
	GOTO	AE8B
	MOVLB	0x0
	BSF	PORTD, 0x2
	GOTO	A68D
AE8B:
	MOVLB	0x0
	BCF	PORTD, 0x2
A68D:	MOVF	button, W	; LCD_D5 = (n >> 1) & 0x01;
	MOVWF	__pcstackCOMMON
	MOVLW	0x1
AE90:	
	LSRF	__pcstackCOMMON, F
	DECFSZ	WREG, F
	GOTO	AE90
	BTFSS	__pcstackCOMMON, 0x0
	GOTO	AE98
	MOVLB	0x0
	BSF	PORTD, 0x3
	GOTO	AE9A
AE98:
	MOVLB	0x0
	BCF	PORTD, 0x3

AE9A:
	MOVF	button, W	; LCD_D6 = (n >> 2) & 0x01;
	MOVWF	__pcstackCOMMON
	MOVLW	0x2
AE9D:
	LSRF	__pcstackCOMMON, F
	DECFSZ	WREG, F
	GOTO	AE9D
	BTFSS	__pcstackCOMMON, 0x0
	GOTO	AEA5
	MOVLB	0x0
	BSF	PORTD, 0x4
	gOTO	AEA7
AEA5:	MOVLB	0x0
	BCF	PORTD, 0x4

AEA7:	; LCD_D7 = (n >> 3) & 0x01;
	MOVF	button, W
	MOVWF	__pcstackCOMMON
	MOVLW	0x3
AEAA:
	LSRF	__pcstackCOMMON, F
	DECFSZ	WREG, F
	GOTO	AEAA
	BTFSS	__pcstackCOMMON, 0x0
	GOTO	AEB2
	MOVLB	0x0
	BSF	PORTD, 0x5
	GOTO	AEB4
AEB2:	
	MOVLB	0x0
	BCF	PORTD, 0x5       
 
AEB4:	; LCD_EN = 0; send enable pulse
	BCF	PORTD, 0x1
	nop	;__delay_us(1);          
	MOVLB	0x0	 ;LCD_EN = 1;
	BSF	PORTD, 0x1
	nop	;__delay_us(1);           
	MOVLB	0x0	;LCD_EN = 0;
	BCF	PORTD, 0x1
	CALL	wait245ms	;__delay_ms(50);  
	RETURN
	
;*******************************************************************************	
;	void LCD_PutC(char LCD_Char) {
;*******************************************************************************	
	
LCD_PutC:           
	MOVWF	i             
	MOVLB	0x0	;RS = 1;
	BSF	PORTD, 0x0
	MOVF	i, W	; LCD_Write_Nibble(LCD_Char >> 4);
	MOVWF	count
	MOVLW	0x4
A4EE:
	LSRF	count, F
	DECFSZ	WREG, F
	GOTO	A4EE
	MOVF	count, W
	CALL	LCD_Write_Nibble
	MOVF	i, W	; LCD_Write_Nibble(LCD_Char);
	CALL	LCD_Write_Nibble
	RETURN
;*******************************************************************************	
;            void LCD_Print(char* LCD_Str) {
;*******************************************************************************	
LCD_Print:
	CLRF	number ;uint8_t i = 0;          
	MOVLB	0x0	;RS = 1;
	BSF	PORTD, 0x0
; while (LCD_Str[i] != '\0') {
AE46:	MOVF	i, W
	MOVWF	col
	MOVF	count, W
	MOVWF	row
	MOVF	number, W
	ADDWF	row, W
	MOVWF	FSR0L
	MOVF	col, W
	BTFSC	STATUS, 0x0
	INCF	col, W
	MOVWF	FSR0H
	MOVIW	FSR0++
	BTFSC	STATUS, 0x2
	RETURN
	GOTO	AE46
; LCD_Write_Nibble(LCD_Str[i] >> 4);
	MOVF	i, W
	MOVWF	col
	MOVF	count, W
	MOVWF	row
	MOVF	number, W
	ADDWF	row, W
	MOVWF	FSR0L
	MOVF	col, W
	BTFSC	STATUS, 0x0
	INCF	col, W
	MOVWF	FSR0H
	MOVF	INDF0, W
	MOVWF	sign
	MOVLW	0x4
A662:	
	LSRF	sign, F
	DECFSZ	WREG, F
	GOTO	A662
	MOVF	sign, W
	CALL	LCD_Write_Nibble
; LCD_Write_Nibble(LCD_Str[i++]);
	MOVF	i, W
	MOVWF	col
	MOVF	count, W
	MOVWF	row
	MOVF	number, W
	ADDWF	row, W
	MOVWF	FSR0L
	MOVF	col, W
	BTFSC	STATUS, 0x0
	INCF	col, W
	MOVWF	FSR0H
	MOVF	INDF0, W
	CALL	LCD_Write_Nibble
	MOVLW	0x1
	MOVWF	row
	MOVF	row, W
	ADDWF	number, F
	GOTO	AE46
;Mainloop	
	
A062:	
	MOVLB	0x0
	BTFSC	PORTB, 0x1	; if(!button1); if button B1 is pressed
	GOTO	A16A 
	MOVLW	0x1		; if(debounce(1)) call debounce function (make sure if B1 is pressed)
	CALL	debounce
	BTFSS	STATUS, 0x0
	GOTO	A16A 
	MOVLB	0x0
	CLRF	i		; i = 0;
	CLRF	__pcstackBANK0	; hour = edit(1, 1, hour)= edit (x,y,parameter)
	INCF	__pcstackBANK0, F;y = 1
	MOVF	hour, W
	MOVWF	0x63		; =hour
	MOVF	0x63, W
	MOVWF	al		; =hour
	MOVLW	0x1
	CALL	edit
	MOVLB	0x0
	MOVWF	0x64	
	MOVF	0x64, W
	MOVWF	hour
	CLRF	__pcstackBANK0	; minute = edit(4, 1, minute);
	INCF	__pcstackBANK0, F
	MOVF	minute, W
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	al
	MOVLW	0x4
	CALL	edit
	MOVLB	0x0
	MOVWF	0x64
	MOVF	0x64, W
	MOVWF	minute       

A089:	; while(debounce(1)); call debounce function (wait for button B1 to be released)
	MOVLW	0x1
	CALL	debounce
	BTFSS	STATUS, 0x0
	GOTO	A0AA
	GOTO	A089        

A0AA:	; while(!button2) if button RB2 button is pressed
	MOVLB	0x0
	BTFSS	PORTB, 0x2
	GOTO	A090

A090:	; w_day++; increment day
	MOVLW	0x1
	MOVLB	0x0
	MOVWF	0x63
	MOVF	0x63, W
	ADDWF	w_day, F
	MOVLW	0x8	; if(w_day > 7) w_day = 1;
	SUBWF	w_day, W
	BTFSS	STATUS, 0x0
	GOTO	A09B
	CLRF	w_day
	INCF	w_day, F
;
A09B:	; w_day_display()
	CALL	w_day_display
	CALL	wait5ms	;__delay_ms(200);

	MOVLW	0x2	; LCD_Goto(1, 2);
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	row
	MOVLW	0x1
	CALL	LCD_Goto
	MOVLW	0x0	; LCD_Print("   "); print 3 spaces
	MOVWF	count
	MOVLW	0x90
	MOVWF	i
	CALL	LCD_Print
	CALL	blink		; blink(); call blink function
	CALL	w_day_display	; w_day_display();
	CALL	blink		; blink(); call blink function
	MOVLB	0x0		; if(!button1) if button B1 is pressed
	BTFSC	PORTB, 0x1
	GOTO	A0AA
	MOVLW	0x1		
	CALL	debounce	; if(debounce(1)) debounce function (make sure if B1 is pressed)
	BTFSS	STATUS, 0x0
	GOTO	A0AA
	MOVLW	0x2	; m_day  = edit(5,  2, m_day);
	MOVLB	0x0
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	__pcstackBANK0
	MOVF	m_day, W
	MOVWF	0x64
	MOVF	0x64, W
	MOVWF	al
	MOVLW	0x5
	CALL	edit
	MOVLB	0x0
	MOVWF	0x65
	MOVF	0x65, W
	MOVWF	m_day
	MOVLW	0x2	; month  = edit(8,  2, month);
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	__pcstackBANK0
	MOVF	month, W
	MOVWF	0x64
	MOVF	0x64, W
	MOVWF	al
	MOVLW	0x8
	CALL	edit
	MOVLB	0x0
	MOVWF	0x65
	MOVF	0x65, W
	MOVWF	month
	MOVLW	0x2	; year   = edit(13, 2, year);
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	__pcstackBANK0
	MOVF	year, W
	MOVWF	0x64
	MOVF	0x64, W
	MOVWF	al
	MOVLW	0xD
	CALL	edit
	MOVLB	0x0
	MOVWF	0x65
	MOVF	0x65, W
	MOVWF	year

A0FF:	; while(debounce(1)); call debounce function (wait for button B1 to be released)
	MOVLW	0x1
	CALL	debounce
	BTFSC	STATUS, 0x0
	GOTO	A0FF
	MOVLB	0x0	 ; minute = decimal_to_bcd(minute);
	MOVF	minute, W
	CALL	decimal_to_bcd
	MOVLB	0x0
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	minute
	MOVF	hour, W	; hour   = decimal_to_bcd(hour);
	CALL	decimal_to_bcd
	MOVLB	0x0
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	hour
	MOVF	m_day, W	; m_day  = decimal_to_bcd(m_day);
	CALL	decimal_to_bcd
	MOVLB	0x0
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	m_day
	MOVF	month, W	; month  = decimal_to_bcd(month);
	CALL	decimal_to_bcd
	MOVLB	0x0
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	month

	MOVF	year, W		; year   = decimal_to_bcd(year);
	CALL	decimal_to_bcd
	MOVLB	0x0
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	year        

	; write data to DS3231 RTC

	CALL	I2C_Start
	MOVLW	0xD0
	CALL	I2C_Write	; RTC chip address       
	MOVLW	0x0
	CALL	I2C_Write	; send register address       
	MOVLW	0x0
	CALL	I2C_Write	; reset seconds and start oscillator  
	MOVLB	0x0
	MOVF	minute, W
	CALL	I2C_Write	; write minute value to RTC chip     
	MOVLB	0x0
	MOVF	hour, W
	CALL	I2C_Write	; write hour value to RTC chip
	MOVLB	0x0
	MOVF	w_day, W	
	CALL	I2C_Write	; write day of the week value to RTC chip   
	MOVLB	0x0
	MOVF	m_day, W
	CALL	I2C_Write	; write date (day of the month) value to RTC chip   
	MOVLB	0x0
	MOVF	month, W
	CALL	I2C_Write	; write month value to RTC chip    
	MOVLB	0x0
	MOVF	year, W
	CALL	I2C_Write	; write year value to RTC chip
	CALL	I2C_Stop         ; stop I2C
	CALL	WAIT	   ;__delay_ms(200);

A16A:	; if(!button3) if RB3 button is pressed
	MOVLB	0x0
	BTFSC	PORTB, 0x3
	GOTO	A248
	MOVLW	0x3; if(debounce(3)) call debounce function (make sure if B3 is pressed)
	CALL	debounce
	BTFSS	STATUS, 0x0
	GOTO	A248
	MOVLW	0x5	; i = 5;
	MOVLB	0x0
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	i
	MOVLW	0x3	; alarm1_hour   = edit(5, 3, alarm1_hour);
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	__pcstackBANK0
	MOVF	alarm1_hour, W
	MOVWF	0x64
	MOVF	0x64, W
	MOVWF	al
	MOVLW	0x5
	CALL	edit
	MOVLB	0x0
	MOVWF	0x65
	MOVF	0x65, W
	MOVWF	alarm1_hour
	MOVLW	0x3	; alarm1_minute = edit(8, 3, alarm1_minute);
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	__pcstackBANK0
	MOVF	alarm1_minute, W
	MOVWF	0x64
	MOVF	0x64, W
	MOVWF	al
	MOVLW	0x8
	CALL	edit
	MOVLB	0x0
	MOVWF	0x65
	MOVF	0x65, W
	MOVWF	alarm1_minute
	MOVLW	0x3	; alarm1_status = on_off(18, 3, alarm1_status);
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	__pcstackBANK0
	MOVF	alarm1_status, W
	MOVWF	0x64
	MOVF	0x64, W
	MOVWF	al
	MOVLW	0x12
	CALL	A6D2
	MOVLB	0x0
	MOVWF	0x65
	MOVF	0x65, W
	MOVWF	alarm1_status
	MOVLW	0x5	; i = 5;
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	i
	MOVLW	0x4	; alarm2_hour   = edit(5, 4, alarm2_hour);
	MOVWF	0x63	
	MOVF	0x63, W
	MOVWF	__pcstackBANK0
	MOVF	alarm2_hour, W
	MOVWF	0x64	
	MOVF	0x64, W
	MOVWF	al	
	MOVLW	0x5
	CALL	edit
	MOVLB	0x0
	MOVWF	0x65
	MOVF	0x65, W
	MOVWF	alarm2_hour
	MOVLW	0x4	; alarm2_minute = edit(8, 4, alarm2_minute);
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	__pcstackBANK0
	MOVF	alarm2_minute, W
	MOVWF	0x64
	MOVF	0x64, W
	MOVWF	al
	MOVLW	0x8
	CALL	edit
	MOVLB	0x0
	MOVWF	0x65
	MOVF	0x65, W
	MOVWF	alarm2_minute
	MOVLW	0x4	; alarm2_status = on_off(18, 4, alarm2_status);
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	__pcstackBANK0
	MOVF	alarm2_status, W
	MOVWF	0x64
	MOVF	0x64, W
	MOVWF	al
	MOVLW	0x12
	CALL	A6D2
	MOVLB	0x0
	MOVWF	0x65
	MOVF	0x65, W
	MOVWF	alarm2_status

A1DC:	; while(debounce(3)); call debounce function (wait for button B3 to be released)
	MOVLW	0x3
	CALL	debounce
	BTFSC	STATUS, 0x0
	GOTO	A1DC
	
; convert decimal to BCD

	MOVLB	0x0	; alarm1_minute = decimal_to_bcd(alarm1_minute);
	MOVF	alarm1_minute, W
	CALL	decimal_to_bcd
	MOVLB	0x0
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	alarm1_minute
	MOVF	alarm1_hour, W	; alarm1_hour   = decimal_to_bcd(alarm1_hour);
	CALL	decimal_to_bcd
	MOVLB	0x0
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	alarm1_hour
	MOVF	alarm2_minute, W; alarm2_minute = decimal_to_bcd(alarm2_minute);
	CALL	decimal_to_bcd
	MOVLB	0x0
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	alarm2_minute
	MOVF	alarm2_hour, W	; alarm2_hour   = decimal_to_bcd(alarm2_hour);
	CALL	decimal_to_bcd
	MOVLB	0x0
	MOVWF	0x63
	MOVF	0x63, W
	MOVWF	alarm2_hour
; end conversion
; write alarms data to DS3231
	CALL	I2C_Start
	MOVLW	0xD0
	CALL	I2C_Write	; RTC chip address  
	MOVLW	0x7
	CALL	I2C_Write	; send register address (alarm1 seconds)
	MOVLW	0x0
	CALL	I2C_Write	 ; write 0 to alarm1 seconds
	MOVLB	0x0
	MOVF	alarm1_minute, W
	CALL	I2C_Write	; write alarm1 minutes
	MOVLB	0x0
	MOVF	alarm1_hour, W
	CALL	I2C_Write	; write alarm1 hours
	MOVLW	0x80
	CALL	I2C_Write	; alarm1 when hours, minutes, and seconds match
	MOVLB	0x0
	MOVF	alarm2_minute, W
	CALL	I2C_Write	; write alarm2 minutes
	MOVLB	0x0
	MOVF	alarm2_hour, W
	CALL	I2C_Write	; write alarm2 hours
	MOVLW	0x80
	CALL	I2C_Write	; alarm2 when hours and minutes match
; I2C_Write(4 | alarm1_status | (alarm2_status << 1)); write data to control register (enable interrupt when alarm)
	MOVLB	0x0
	LSLF	alarm2_status, W
	IORWF	alarm1_status, W
	IORLW	0x4
	CALL	I2C_Write    
	MOVLW	0x0
	CALL	I2C_Write	; clear alarm flag bits
	CALL	I2C_Stop
	CALL	wait5ms	; __delay_ms(200);  // wait 200ms

A248:	; if(!button2 && LED_PIN){ when button B2 pressed with alarm (Reset and turn OFF the alarm)
	MOVLB	0x0
	BTFSS	PORTB, 0x2
	BTFSS	PORTB, 0x4
	GOTO	A27C
	BCF	PORTB, 0x4	; LED_PIN = 0; turn OFF alarm indicator
	CALL	I2C_Start         
	MOVLW	0xD0
	CALL	I2C_Write	; RTC chip address       
	MOVLW	0xE
	CALL	I2C_Write	; send register address (control register)
; write data to control register (Turn OFF the occurred alarm and keep the other as it is)
; I2C_Write(4 | ((!(status_reg & 1)) & alarm1_status) | (((!((status_reg >> 1) & 1)) & alarm2_status) << 1));
	MOVLB	0x0
	MOVF	status_reg, W
	MOVWF	0x63
	MOVLW	0x1
A25C:	
	LSRF	0x63, F
	DECFSZ	WREG, F
	GOTO	A25C
	BTFSC	0x63, 0x0
	GOTO	A263
	MOVLW	0x1
	GOTO	A264
A263:	
	MOVLW	0x0
A264:	
	ANDWF	alarm2_status, W
	MOVWF	0x64
	MOVLW	0x1
A267:	
        LSLF	0x64, F
	DECFSZ	WREG, F
	GOTO	A267
	BTFSC	status_reg, 0x0
	GOTO	A26E
	MOVLW	0x1
	GOTO	A26F
A26E:	
	MOVLW	0x0
A26F:	
	ANDWF	alarm1_status, W
	IORWF	0x64, W
	IORLW	0x4
	CALL	I2C_Write    
	MOVLW	0x0
	CALL	I2C_Write	; clear alarm flag bits
	CALL	I2C_Stop
A27C:	
	CALL	RTC_read	; read time and date from RTC chip
	CALL	alarms_read_display	; read and display alarms
	CALL	RTC_display	; print time & date
	CALL	wait5ms	;__delay_ms(50);   // wait 50 ms
	GOTO	A062	
	
	end	resetVec